#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <getopt.h>
#include "wavplay.h"
#include "dmcconv.h"

// prototype define -----------
int check_opt(int argc, char *argv[]);
char *basename(char* argv1, char* argv2);
void usage(char *appname);
void header();
extern void errmes(char *str);

// ----------------------------
int main( int argc, char *argv[] )
{
	WAVFILE	*wfile;
	int		cnt = 0;
	WORD	dat;
	u_long	nDataSize;
	double	SampleStep;
	LONG 	nSampleSize;

	int	DMCtbl[] = {
		0xD60, 0xBE0, 0xAA0, 0xA00, 0x8F0, 0x7F0, 0x710, 0x6B0,
		0x5F0, 0x500, 0x470, 0x400, 0x350, 0x2A0, 0x240, 0x1B0,
	};

	DMCrate = 0x0F;
	nVolume = 100;
	bVolume = true;
	bSize   = true;
	bPadding = false;
	pInFile = NULL;
	pOutFile = NULL;

	if(check_opt(argc, argv)) return -1;

	wfile = WavOpenForRead(pInFile, NULL);
	if(wfile == NULL) errmes("ERROR: not wav format file.");

#ifdef DEBUG
	printf(" --channel      : %d\n", wfile->wavinfo.Channels);
	printf(" --sampling rate: %d\n", wfile->wavinfo.SamplingRate);
	printf(" --sample bits  : %d\n", wfile->wavinfo.DataBits);
	printf(" --samples      : %d\n", wfile->wavinfo.Samples);
	printf(" --data start   : %d\n", wfile->wavinfo.DataStart);
	printf(" --data bytes   : %d\n", wfile->wavinfo.DataBytes);
#endif

	nDataSize = wfile->wavinfo.DataBytes;
	if( wfile->wavinfo.Channels != Mono )
		errmes("ERROR: support only mono format.");
	if( wfile->wavinfo.DataBits != 16 )
		errmes("ERROR: support only 16bit format.");
	if(!(lpBuffer = (Byte*)malloc( nDataSize )))
		errmes("ERROR: memory allocate error.");

	if(lseek(wfile->fd, wfile->wavinfo.DataStart, SEEK_SET)
		!= wfile->wavinfo.DataStart) errmes("ERROR: file seek error.");
	if(read(wfile->fd, lpBuffer, nDataSize) != nDataSize)
		errmes("ERROR: file read error.");

	nSampleSize = nDataSize/(wfile->wavinfo.DataBits/8);
	// ץ󥰥졼ȤDMCץ󥰥졼ȤΥƥå
	SampleStep = ((1789772.5*8)/(double)DMCtbl[DMCrate])/(double)wfile->wavinfo.SamplingRate;

	printf( "Wave File SamplingRate: %dHz\n", wfile->wavinfo.SamplingRate );
	printf( "DMC       SamplingRate: %7.3fHz\n", ((1789772.5*8)/(double)DMCtbl[DMCrate]) );
	printf( "Smaple size: %d samples\n", nSampleSize );

	DMCconv(SampleStep, nSampleSize);

	free(lpBuffer);

	return 0;
}

int check_opt(int argc, char *argv[])
{
	header();

	while(1){
		int c;
		int option_index = 0;
		static struct option long_options[] = {
			{"sampling-rate",		1, 0, 'r'},
			{"volume",				1, 0, 'v'},
			{"volume-not-adjust",	0, 0, 'n'},
			{"banksize-padding",	0, 0, 'b'},
			{"size-correct",		0, 0, 's'},
			{"help",				0, 0, 'h'},
			{0, 0, 0, 0}
		};
		c = getopt_long(argc, argv, "r:v:nbsh", long_options, &option_index);
		if(c == EOF) break;

		switch(c) {
			case 'r':
				DMCrate = strtol(optarg, NULL, 16);
				break;
			case 'v':
				nVolume = atoi(optarg);
				bVolume = true;
				break;
			case 'b':
				bPadding = true;
				break;
			case 'n':
				bVolume = false;
				break;
			case 's':
				bSize = false;
				break;
			case 'h':
			default:
				usage(argv[0]);
				break;
		}
	}

	if(!argv[optind]){
		printf( "ERROR: require input file.\n" );
		usage(argv[0]);
	}
	pInFile  = argv[optind++];
	if(!argv[optind]){
		pOutFile = basename(argv[optind-1], ".wav");
		strcat(pOutFile, ".dmc");
		printf("outfile: %s\n", pOutFile);
	} else
		pOutFile = argv[optind++];
	if( DMCrate < 0 || DMCrate > 0x0F )
		errmes( "ERROR: sampling rate must be 0-F.");

	return 0;
}

char *basename( char *argv1, char *argv2 )
{
    char *ptr;
    char *bname, *basename;

    bname = NULL;

    // error check
    if(argv1 == NULL ||
        argv2 != NULL && strlen(argv1) < strlen(argv2)) return NULL;

    // get basename
    ptr = argv1 + strlen(argv1);
    while(ptr--!=argv1){
        if(*ptr == '/'){
            break;
        }
    }
	bname = ptr+1;
	basename = (char *)malloc(strlen(bname)+1);
	strcpy(basename, bname);

    // error check
    if(argv2 == NULL || strlen(basename) < strlen(argv2)) return basename;
    ptr = basename + strlen(basename) - strlen(argv2);
    if(!strcmp(ptr, argv2))
        memset(ptr,'\0',strlen(argv2));

    return basename;
}

void header()
{
	printf( "DMC sample converter for arm-linux v0.01 programmed by erio\n" );
	printf( "original DMC sample converter v0.05 programmed by Norix\n" );
	printf( "original wavplay v1.4 programmed by Warren W. Gay VE3WWG\n" );
	return;
}

void usage(char *appname)
{
	printf( "Usage: %s [option] wavefile [outfile]\n", appname );
	printf( " Options\n" );
	printf( " -r n DMC Sampling rate(0-F) (Default:F 33.14KHz)\n" );
	printf( "      0: 4.18KHz  1: 4.71KHz  2: 5.26KHz  3: 5.59KHz\n" );
	printf( "      4: 6.26KHz  5: 7.05KHz  6: 7.92KHz  7: 8.36KHz\n" );
	printf( "      8: 9.42KHz  9:11.18KHz  A:12.60KHz  B:13.98KHz\n" );
	printf( "      C:16.88KHz  D:21.30KHz  E:24.86KHz  F:33.14KHz\n" );
	printf( " -v n Volume(Default:100) \n" );
	printf( " -n   Volume not adjust(Default:Adjust)\n" );
	printf( " -b   Bank size padding(Default:No padding)\n" );
	printf( " -s   Size correct(Default:Correct)\n" );
	exit(1);
}

void errmes(char *str)
{
	printf(str);
	printf("\n");
	exit(-1);
}
